/*
** Module   : EnvironmentDaemon.java
** Abstract : Client side daemon to handle Progress 4GL compatible Winodws environment handling
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130604 Created initial version.
*/

package com.goldencode.p2j.util;

import java.util.*;

import com.goldencode.p2j.net.*;

public class EnvironmentDaemon
implements EnvironmentAccessor
{
   /** Token used to authenticate with the dispatcher when registering APIs. */
   private static Object modToken = null;
   
   private EnvironmentAccessor defaultEnv = null;
   
   private EnvironmentAccessor currentEnv = null;
   
   private Map<String, EnvironmentAccessor> envMap = new HashMap<String, EnvironmentAccessor>();
   
   /**
    * Create an instance and export its API to the network.
    *
    * @param    single
    *           <code>true</code> to startup within the server process which
    *           must bypass the shared infrastructure initialization. Use
    *           <code>false</code> for the normal client JVM startup.
    */
   public EnvironmentDaemon(boolean single)
   {
      defaultEnv = new Registry();
      
      synchronized (EnvironmentDaemon.class)
      {
         modToken = RemoteObject.registerServer(EnvironmentDaemon.class,
                                                this,
                                                modToken,
                                                single);
      }
   }
      
   /**
    * Creates application defaults implementing Progress LOAD statement.
    * 
    * @param env 
    *        The environment to create or load. Can be the registry key,
    *        initialization file.
    * @param directory 
    *        The directory path for existing initialization file or place
    *        to create initialization file.
    * @param flagNew
    *        If <code>true</code> - the new file or registry key will be
    *        created overwriting the possibly existed one.
    * @param baseKey
    *        When the parameter is specified the serching is performed under
    *        the particular key value. It is possible to specify "INI" as
    *        parameter. In this case search or creation will be dome inside
    *        initialization file specified in env parameter.
    */
   public void load(character env, character directory, logical flagNew, character baseKey)
   {
      // TODO implement this
      EnvironmentAccessor envAcc = null;
      if (baseKey.toStringMessage().equals("INI"))
      {
         envAcc = new StanzaIni(env, directory, flagNew);
      }
      else
      {
         envAcc = new Registry(env, flagNew, baseKey);
      }
      envAcc.load(env, directory, flagNew, baseKey);
      envMap.put(env.toStringMessage().toLowerCase(), envAcc);
   }

   /**
    * Specifies environment defaults to use in subsequent windows.
    * 
    * @param env 
    *        The environment to load. If the parameter is null string("") the
    *        default environment becomes the current one.
    */
   public void use(character env)
   {
      // TODO implement this
      currentEnv = envMap.get(env.toStringMessage().toLowerCase());
   }

   /**
    * Adds, modifies and deletes keys in current environment.
    * 
    * @param section 
    *        The name of the section containing the key to modify.
    * @param key 
    *        The name of the key key to modify or default key if not specified.
    * @param value 
    *        The new value of the key under modification.
    */
   public void setKeyValue(character section, character key, character value)
   {
      // TODO implement this
      if (currentEnv != null)
      {
         currentEnv.setKeyValue(section, key, value);
      }
      else
      {
         defaultEnv.setKeyValue(section, key, value);
      }
   }

   /**
    * Getting the key value from the current environment.
    * 
    * @param section 
    *        The name of the section containing the key to get.
    * @param key 
    *        The name of the key key to get or default key if not specified.
    * 
    * @return The value of the key.
    */
   public character getKeyValue(character section, character key)
   {
      // TODO implement this
      if (currentEnv != null)
      {
         return currentEnv.getKeyValue(section, key);
      }
      else
      {
         return defaultEnv.getKeyValue(section, key);
      }
   }

   /**
    * Unloads specifies environment from the current one.
    * 
    * @param env 
    *        The environment to unload. Can be the registry or initialization
    *        file. The environment to unload should be previously loaded by
    *        load() method.
    */
   public void unload(character env)
   {
      // TODO implement this
      if (currentEnv != null)
      {
         currentEnv = null;
      }
      return;
   }
}
