/*
** Module   : Environments.java
** Abstract : Progress 4GL compatible Winodws registry environments related methods
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130522 Created initial version.
*/

package com.goldencode.p2j.util;

import com.goldencode.p2j.net.*;
import com.goldencode.p2j.security.ContextLocal;

public class Environments
{
   /** Stores context-local state variables. */
   private static ContextContainer work = new ContextContainer();
   
   /**
    * Creates application defaults implementing Progress LOAD statement.
    * 
    * @param env 
    *        The environment to create or load. Can be the registry key,
    *        initialization file.
    * @param directory 
    *        The directory path for existing initialization file or place
    *        to create initialization file.
    * @param flagNew
    *        If <code>true</code> - the new file or registry key will be
    *        created overwriting the possibly existed one.
    * @param baseKey
    *        When the parameter is specified the serching is performed under
    *        the particular key value. It is possible to specify "INI" as
    *        parameter. In this case search or creation will be dome inside
    *        initialization file specified in env parameter.
    */
   public static void load(character env, character directory, logical flagNew, character baseKey)
   {
      // TODO implement this
      work.obtain().envAcc.load(env, directory, flagNew, baseKey);
   }

   /**
    * Specifies environment defaults to use in subsequent windows.
    * 
    * @param env 
    *        The environment to load. If the parameter is null string("") the
    *        default environment becomes the current one.
    */
   public static void use(character env)
   {
      // TODO implement this
      work.obtain().envAcc.use(env);
   }

   /**
    * Adds, modifies and deletes keys in current environment.
    * 
    * @param section 
    *        The name of the section containing the key to modify.
    * @param key 
    *        The name of the key key to modify or default key if not specified.
    * @param value 
    *        The new value of the key under modification.
    */
   public static void setKeyValue(character section, character key, character value)
   {
      // TODO implement this
      work.obtain().envAcc.setKeyValue(section, key, value);
   }

   /**
    * Getting the key value from the current environment.
    * 
    * @param section 
    *        The name of the section containing the key to get.
    * @param key 
    *        The name of the key key to get or default key if not specified.
    * 
    * @return The value of the key.
    */
   public static character getKeyValue(character section, character key)
   {
      // TODO implement this
      return work.obtain().envAcc.getKeyValue(section, key);
   }

   /**
    * Unloads specifies environment from the current one.
    * 
    * @param env 
    *        The environment to unload. Can be the registry or initialization
    *        file. The environment to unload should be previously loaded by
    *        load() method.
    */
   public static void unload(character env)
   {
      // TODO implement this
      work.obtain().envAcc.unload(env);
   }

   /** 
    * Stores global data relating to the state of the current context.
    */
   private static class WorkArea
   {  
      /** Stores the Progress error code for the last operation. */ 
//      private int lastError = 0; 
      
      /** Remote instance of our file system worker. */ 
      private EnvironmentAccessor envAcc = null; 
   }
   
   /**
    * Simple container that stores and returns a context-local instance of
    * the global work area.
    */
   private static class ContextContainer
   extends ContextLocal
   { 
      /**
       * Obtains the context-local instance of the contained global work
       * area.
       *
       * @return   The work area associated with this context.
       */
      public WorkArea obtain() 
      {
         return (WorkArea) this.get();
      }   
      
      /**
       * Initializes the work area, the first time it is requested within a
       * new context.
       *
       * @return   The newly instantiated work area.
       */
      protected synchronized Object initialValue()
      {
         WorkArea wa = new WorkArea();
         
         // obtain the our proxy, server-side code is left-side by convention
         wa.envAcc = (EnvironmentAccessor) RemoteObject.obtainInstance(EnvironmentAccessor.class,
                                                                       true);
         
         return wa;
      }   
   }   
}
