/*
** Module   : Registry.java
** Abstract : Progress 4GL compatible Winodws registry related methods
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130604 Created initial version.
*/

package com.goldencode.p2j.util;

import com.goldencode.util.*;

public class Registry
implements EnvironmentAccessor
{
   /** Windows registry key-subkey names separator. */
   public static final String NAME_SEPARATOR = "\\\\";
   
   private native static String getKeyVal(String base, String section, String key);
   
   private native static boolean setKeyVal(String base, String section, String key, String value);

   static
   {
      if (PlatformHelper.isUnderWindowsFamily())
      {
         System.loadLibrary("p2j");
      }
   }

   private String registryKeyName = null;
   
   private String baseKeyName = null;
   
   private boolean isNew;

   public Registry()
   {
      registryKeyName = "SOFTWARE"+NAME_SEPARATOR+"PSC"+NAME_SEPARATOR+"PROGRESS"+NAME_SEPARATOR+
                        EnvironmentOps.getVersion().toStringMessage();
      isNew = false;
      baseKeyName = "HKEY_CURRENT_USER";
   }

   public Registry(character env, logical flagNew, character baseKey)
   {
      registryKeyName = env.toStringMessage();
      isNew = flagNew.booleanValue(); 
      baseKeyName = baseKey.toStringMessage();
   }

   /**
    * Creates application defaults implementing Progress LOAD statement.
    * 
    * @param env 
    *        The environment to create or load. Can be the registry key,
    *        initialization file.
    * @param directory 
    *        The directory path for existing initialization file or place
    *        to create initialization file.
    * @param flagNew
    *        If <code>true</code> - the new file or registry key will be
    *        created overwriting the possibly existed one.
    * @param baseKey
    *        When the parameter is specified the serching is performed under
    *        the particular key value. It is possible to specify "INI" as
    *        parameter. In this case search or creation will be dome inside
    *        initialization file specified in env parameter.
    */
   public void load(character env, character directory, logical flagNew, character baseKey)
   {
      // TODO implement this
      return;
   }

   /**
    * Specifies environment defaults to use in subsequent windows.
    * 
    * @param env 
    *        The environment to load. If the parameter is null string("") the
    *        default environment becomes the current one.
    */
   public void use(character env)
   {
      // TODO implement this
      return;
   }

   /**
    * Adds, modifies and deletes keys in current environment.
    * 
    * @param section 
    *        The name of the section containing the key to modify.
    * @param key 
    *        The name of the key key to modify or default key if not specified.
    * @param value 
    *        The new value of the key under modification.
    */
   public void setKeyValue(character section, character key, character value)
   {
      // TODO implement this
      setKeyVal(baseKeyName, registryKeyName + NAME_SEPARATOR + section.toStringMessage(),
                key.toStringMessage(), value.toStringMessage());
   }

   /**
    * Getting the key value from the current environment.
    * 
    * @param section 
    *        The name of the section containing the key to get.
    * @param key 
    *        The name of the key key to get or default key if not specified.
    * 
    * @return The value of the key.
    */
   public character getKeyValue(character section, character key)
   {
      // TODO implement this
      String result = getKeyVal(baseKeyName,
                                registryKeyName + NAME_SEPARATOR + section.toStringMessage(),
                                key.toStringMessage());
      return new character(result);
   }

   /**
    * Unloads specifies environment from the current one.
    * 
    * @param env 
    *        The environment to unload. Can be the registry or initialization
    *        file. The environment to unload should be previously loaded by
    *        load() method.
    */
   public void unload(character env)
   {
      // TODO implement this
      return;
   }
}
