/*
** Module   : StanzaIni.java
** Abstract : Progress 4GL compatible Winodws INI files related methods
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130604 Created initial version.
*/

package com.goldencode.p2j.util;

import java.io.*;
import java.util.*;

public class StanzaIni
implements EnvironmentAccessor
{
   private String name = null;
   
   private String workingDirectory = null;
   
   private boolean isNew = false;

   private Map<character, Map<character, character>> envMap = null;
   
   public StanzaIni()
   {
      envMap = new HashMap<character, Map<character, character>>();
   }
   
   public StanzaIni(character env, character directory, logical flagNew)
   {
      this();

      name = env.toStringMessage().toLowerCase() + ".ini";
      workingDirectory = directory.toStringMessage().toLowerCase();
      isNew = flagNew.booleanValue();
   }

   /**
    * Creates application defaults implementing Progress LOAD statement.
    * 
    * @param env 
    *        The environment to create or load. Can be the registry key,
    *        initialization file.
    * @param directory 
    *        The directory path for existing initialization file or place
    *        to create initialization file.
    * @param flagNew
    *        If <code>true</code> - the new file or registry key will be
    *        created overwriting the possibly existed one.
    * @param baseKey
    *        When the parameter is specified the serching is performed under
    *        the particular key value. It is possible to specify "INI" as
    *        parameter. In this case search or creation will be dome inside
    *        initialization file specified in env parameter.
    */
   public void load(character env, character directory, logical flagNew, character baseKey)
   {
      // TODO implement this
      if (!isNew)
      {
         loadIniFile();
      }
      else
      {
         createEmptyIniFile();
      }
   }

   /**
    * Specifies environment defaults to use in subsequent windows.
    * 
    * @param env 
    *        The environment to load. If the parameter is null string("") the
    *        default environment becomes the current one.
    */
   public void use(character env)
   {
      // TODO implement this
      return;
   }

   /**
    * Adds, modifies and deletes keys in current environment.
    * 
    * @param section 
    *        The name of the section containing the key to modify.
    * @param key 
    *        The name of the key key to modify or default key if not specified.
    * @param value 
    *        The new value of the key under modification.
    */
   public void setKeyValue(character section, character key, character value)
   {
      // TODO implement this
      Map<character, character> sectionBody = envMap.get(section);
      sectionBody.put(key, value);
      
      commitIniFile();
   }

   /**
    * Getting the key value from the current environment.
    * 
    * @param section 
    *        The name of the section containing the key to get.
    * @param key 
    *        The name of the key key to get or default key if not specified.
    * 
    * @return The value of the key.
    */
   public character getKeyValue(character section, character key)
   {
      // TODO implement this
      Map<character, character> sectionBody = envMap.get(section);
      character value = sectionBody.get(key);
      return value;
   }

   /**
    * Unloads specifies environment from the current one.
    * 
    * @param env 
    *        The environment to unload. Can be the registry or initialization
    *        file. The environment to unload should be previously loaded by
    *        load() method.
    */
   public void unload(character env)
   {
      // TODO implement this
      return;
   }

   /**
    * Reads the current environment set from appropriate INI file in file system.
    */
   private void loadIniFile()
   {
      String fileName = "";
      
      if (workingDirectory != null && workingDirectory.length() > 0)
      {
         fileName.concat(workingDirectory+EnvironmentOps.getLegacyFileSeparator());
      }
      if (name != null && name.length() > 0)
      {
         fileName.concat(name);
         
         BufferedReader br = null;
         
         try
         {
            // Open file stream for reading
            br = new BufferedReader(new FileReader(fileName));

            character sectionName = null;
            Map<character, character> sectionBody = null;
            
            String nextLine = br.readLine();
            // Read until EOF(nextLine == null)
            while (nextLine != null)
            {
               // Next INI file section should start here
               if (nextLine.charAt(0) == '[' &&
                   nextLine.charAt(nextLine.length() - 1) == ']')
               {
                  // The previously constructed section should be saved before starting to
                  // process the next section
                  if (sectionName != null && sectionBody != null && sectionBody.size() > 0)
                  {
                     envMap.put(sectionName, sectionBody);
                  }
                  // The new section name candidate
                  sectionName = new character(nextLine.substring(1,nextLine.length() - 2));
                  // Purge the previous one
                  sectionBody.clear();
                  sectionBody = null;
               }
               // Section processing
               else
               {
                  // Key-value candidate, get the index of separator first
                  int separatorNdx = nextLine.indexOf('=');
                  // Something should exist before and after separator
                  if (separatorNdx > 0 && nextLine.length() > separatorNdx + 1)
                  {
                     if (sectionBody == null)
                     {
                        sectionBody = new HashMap<character, character>();
                     }
                     // Construct key-value pair from line string
                     character keyName =
                                    new character(nextLine.substring(0,separatorNdx - 1).trim());
                     character keyValue =
                                    new character(nextLine.substring(separatorNdx + 1).trim());
                     // Store the next  key-value pair into section map
                     sectionBody.put(keyName, keyValue);
                  }
               }
               
               // Read the next line from file
               nextLine = br.readLine();
               if (nextLine == null && sectionName != null &&
                   sectionBody != null && sectionBody.size() > 0)
               {
                  // Possible last section in the file
                  envMap.put(sectionName, sectionBody);
               }
            }
            br.close();
         }
         catch (IOException ioex)
         {
         }
      }
   }
   
   /**
    * Creates the empty INI file for current environment set in file system.
    */
   private void createEmptyIniFile()
   {
      String fileName = "";
      
      if (workingDirectory != null && workingDirectory.length() > 0)
      {
         fileName.concat(workingDirectory+EnvironmentOps.getLegacyFileSeparator());
      }
      if (name != null && name.length() > 0)
      {
         fileName.concat(name);

         PrintWriter pw = null;
         
         try
         {
            // Open file for writing
            pw = new PrintWriter(new FileWriter(fileName, false));
         }
         catch (IOException ioex)
         {
         }
         finally
         {
            // Do nothing and close it
            if (pw != null)
            {
               pw.close();
            }
         }
      }
   }
   
   /**
    * Writes the current environment set to the appropriate INI file in file system.
    */
   private void commitIniFile()
   {
      String fileName = "";
      
      if (workingDirectory != null && workingDirectory.length() > 0)
      {
         fileName.concat(workingDirectory+EnvironmentOps.getLegacyFileSeparator());
      }
      if (name != null && name.length() > 0)
      {
         fileName.concat(name);
         
         PrintWriter pw = null;
         
         try
         {
            // Open file for writing
            pw = new PrintWriter(new FileWriter(fileName, false));
            // Sections iterator
            Iterator iterSect = envMap.keySet().iterator();
            while (iterSect.hasNext())
            {
               // Get next available section name
               character sectionName = (character)iterSect.next();
               Map<character, character> sectionBody = envMap.get(sectionName);
               // If section name is not empty write the section name
               if (sectionBody.size() > 0)
               {
                  pw.println("["+sectionName.toStringMessage()+"]");
               }
               // Key names iterator
               Iterator iterKey = sectionBody.keySet().iterator();
               while (iterKey.hasNext())
               {
                  // Get next available key=value pair
                  character keyName = (character)iterKey.next();
                  character keyValue = sectionBody.get(keyName);
                  // And write them to the file
                  pw.println(keyName.toStringMessage() + "=" + keyValue.toStringMessage());
               }
               // Separate two sections with empty line
               if (sectionBody.size() > 0 && iterSect.hasNext())
               {
                  pw.println();
               }
            }
            // Commit the changes
            pw.flush();
         }
         catch (IOException ioex)
         {
         }
         finally
         {
            if (pw != null)
            {
               // And finally close the file
               pw.close();
            }
         }
      }
   }
}
